//
//  MyWindowController.m
//  OrbBASIC 1.0
//
// Created by Adam Wilson - Sphero Rangers 2013
//

#import "MyWindowController.h"
#import <RobotKit/RobotKit.h>

@implementation MyWindowController


#pragma mark Application Lifecycle


- (id)initWithPath:(NSString *)newPath
{
    /*Register for application lifecycle notifications so we known when to connect and disconnect from the robot*/
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appDidBecomeActive:) name:NSApplicationDidBecomeActiveNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appWillResignActive:) name:NSApplicationWillResignActiveNotification object:nil];

    orbBasicProgram = nil;
    robotstatus     = 0;
    
    return [super initWithWindowNibName:@"Window"];
}

-(void)appDidBecomeActive:(NSNotification*)notification {

}

-(void)appWillResignActive:(NSNotification*)notification {
    
    // Stop observing responses and asynchronous data from Sphero.
    [[RKDeviceMessenger sharedMessenger] removeDataStreamingObserver:self];
    [[RKDeviceMessenger sharedMessenger] removeResponseObserver:self];
    
    // Stop communication with Sphero.
    [messageView    setString:@"!!  Sphero Disconnected !!!\n"];
    [reconnectlabel setStringValue:@" <No Sphero Connected>"];
    robotstatus = 0;
    
    [[RKRobotProvider sharedRobotProvider] closeRobotConnection];
}

- (void)dealloc
{   
	[super dealloc];
}


- (IBAction)printOrbBASICDocument:(id)sender
{
    NSTextView *printView = [[NSTextView alloc] initWithFrame:NSMakeRect(0, 0, 468, 648)];
    [printView setString:[orbbasic string]];
    [printView autorelease];
    
    [[NSPrintOperation printOperationWithView:printView] runOperation];
}

-(IBAction)newPressed:(id)sender{
    [orbbasic setString:[NSString stringWithFormat:@""]];
    [filenamelabel setStringValue:@""];
    orbBasicProgram = nil;
    
    [self clearDebug:nil];
}


#pragma mark Program Setup


-(void)setupFonts{
    [orbbasic setFont:[NSFont fontWithName:@"Menlo" size:18]];
    [orbbasic setContinuousSpellCheckingEnabled:NO];
    
    [messageView setFont:[NSFont fontWithName:@"Menlo" size:12]];
    [messageView setContinuousSpellCheckingEnabled:NO];
    [messageView setBackgroundColor:[NSColor colorWithCalibratedRed:0.0 green:0.0 blue:0.0 alpha:1.0f]];
    [messageView setTextColor:[NSColor colorWithCalibratedRed:0.5 green:0.70 blue:1.0 alpha:1.0f]];
    
    [self setupSliders];
}

-(void)setupSliders{
    [redLabel setStringValue:   @"0"];
    [greenLabel setStringValue: @"255"];
    [blueLabel setStringValue:  @"0"];
}



#pragma mark Robot Communication


-(void)robotWentOnline {
    
    [messageView setString:@" [Sphero Connected Successfully]\n"];
    
    // Set observer methods to handle messages from Sphero.
    [[RKDeviceMessenger sharedMessenger] addResponseObserver:self selector:@selector(handleResponse:)];
    [[RKDeviceMessenger sharedMessenger] addDataStreamingObserver:self selector:@selector(handleAsyncData:)];
    
    // Get Bluetooth and Power information
    [RKGetBluetoothInfoCommand sendCommand];
    [RKGetPowerStateCommand sendCommand];
    
    // reset the state of the ball ie. Stabilization and BackLED and the UI.
    [RKStabilizationCommand sendCommandWithState:RKStabilizationStateOn];
    [stabilizationOnButton  setState:NSOnState];
    [stabilizationOffButton setState:NSOffState];

    [RKBackLEDOutputCommand sendCommandWithBrightness:0.0];
    [backLEDOffButton setState:NSOnState];
    [backLEDOnButton  setState:NSOffState];
    
    robotstatus = 1;
}


-(IBAction)reconnect:(id)sender{
    [[RKRobotProvider sharedRobotProvider] openRobotConnection];    
}


-(void)changeColorWithRed:(int)r Green:(int)g Blue:(int)b{
    [RKRGBLEDOutputCommand sendCommandWithRed:r/255. green :g/255. blue :b/255.];
    
    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  RGB %d, %d, %d\n",[messageView string],r,g,b]];
    [messageView scrollToEndOfDocument:nil];
    
    [rgbColorWell setColor:[NSColor colorWithCalibratedRed:r/255. green:g/255. blue:b/255. alpha:1.0f]];
}

-(void)stopSphero{
    [RKRollCommand sendCommandWithHeading:[directionSlider floatValue] velocity:0.0 stopped:true];
    
    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  goroll %d, %d\n\n" ,[messageView string],[directionSlider intValue],0]];
    [messageView scrollToEndOfDocument:nil];
}


#pragma mark Update UI


-(void)updateColorWell{
    int redValue = [redLabel intValue];
    int greenValue = [greenLabel intValue];
    int blueValue = [blueLabel intValue];
    
    if(robotstatus == 1)[[NSSound soundNamed:@"Tink"] play];
    
    [self changeColorWithRed:redValue Green:greenValue Blue:blueValue];
}

-(IBAction)clearDebug:(id)sender{
    [messageView setString:@""];
}


#pragma mark File Handlers


-(void)openDialogToSave{
    NSSavePanel *save = [NSSavePanel savePanel];
    
    // Only allow our orbbas file types
    NSArray* fileTypes = [[[NSArray alloc] initWithObjects:@"orbbas", @"ORBBAS", nil] autorelease];
    [save setAllowedFileTypes:fileTypes];
    
    long i = [save runModal];
    if(i == 1){
        [messageView setString:[NSString stringWithFormat:@"%@-> Saving....\n",[messageView string]]];
        
        NSString* fullFilePath = save.URL.filePathURL.path;
        NSArray *parts = [fullFilePath componentsSeparatedByString:@"/"];
        NSString *filename = [parts lastObject];
        NSString* filePath = [fullFilePath substringToIndex:([fullFilePath length]-[filename length])];
        
        
        if (![[NSFileManager defaultManager] fileExistsAtPath:fullFilePath]) {
            [[NSFileManager defaultManager] createFileAtPath:fullFilePath contents:nil attributes:nil];
        }
        
        [[[orbbasic string] dataUsingEncoding:NSUTF8StringEncoding] writeToFile:fullFilePath atomically:NO];
        
        [messageView setString:[NSString stringWithFormat:@"%@  ... [%@ Saved]\n",[messageView string],filename]];
        [messageView scrollToEndOfDocument:nil];
        
        // This class object handles loading, erasing, and executing the orbBasic program
        orbBasicProgram = [[RKOrbBasicProgram alloc] initWithFilePath:filename
                                                        directoryPath:filePath
                                                                error:nil];
        
        [filenamelabel setStringValue:filename];
        [filePathlabel setStringValue:filePath];
    }
}

-(IBAction)openPressed:(id)sender {
    NSOpenPanel *panel;
    
    // Only allow our orbbas file types
    NSArray* fileTypes = [[[NSArray alloc] initWithObjects:@"orbbas", @"ORBBAS", nil] autorelease];
    panel = [NSOpenPanel openPanel];
    [panel setFloatingPanel:YES];
    [panel setCanChooseDirectories:NO];
    [panel setCanChooseFiles:YES];
    [panel setAllowsMultipleSelection:NO];
    [panel setAllowedFileTypes:fileTypes];
    
    long i = [panel runModal];
    
    if(i == NSOKButton){
        NSString* fullFilePath = panel.URL.filePathURL.path;
        NSArray *parts = [fullFilePath componentsSeparatedByString:@"/"];
        NSString *filename = [parts lastObject];
        NSString* filePath = [fullFilePath substringToIndex:([fullFilePath length]-[filename length])];
        
        [messageView setString:[NSString stringWithFormat:@"%@-> Opening File: %@\n",[messageView string],filename]];
        [messageView scrollToEndOfDocument:nil];
        
        // This class object handles loading, erasing, and executing the orbBasic program
        orbBasicProgram = [[RKOrbBasicProgram alloc] initWithFilePath:filename
                                                        directoryPath:filePath
                                                                error:nil];
        
        [filenamelabel setStringValue:filename];
        [filePathlabel setStringValue:filePath];
        
        [orbbasic setString: [NSString stringWithContentsOfFile:fullFilePath encoding:NSUTF8StringEncoding error:nil]];
    }
}


-(void)loadCurrentFileIntoOrbBasic{
    NSString* fullFilePath = [[filePathlabel stringValue] stringByAppendingPathComponent:[filenamelabel stringValue]];
    
    [messageView setString:[NSString stringWithFormat:@"%@-> Preparing File: %@\n",[messageView string],[filenamelabel stringValue]]];
    [messageView scrollToEndOfDocument:nil];
    
    // Check to see if it exists.
    if (![[NSFileManager defaultManager] fileExistsAtPath:fullFilePath]) {
        [[NSFileManager defaultManager] createFileAtPath:fullFilePath contents:nil attributes:nil];
    }
    
    // Save everytime you Execute a program.
    [[[orbbasic string] dataUsingEncoding:NSUTF8StringEncoding] writeToFile:fullFilePath atomically:YES];
    orbBasicProgram = [[RKOrbBasicProgram alloc] initWithFilePath:[filenamelabel stringValue]
                                                    directoryPath:[filePathlabel stringValue]
                                                            error:nil];
}


#pragma Real Buttons

-(IBAction)getBatteryInfoPressed:(id)sender{
    
    [RKGetPowerStateCommand sendCommand];
    
    if(robotstatus == 0){
        [messageView setString:[NSString stringWithFormat:@"%@ Connect to your Sphero.   (See help section for Bluetooth Pairing instructions.) \n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
    }
}

-(IBAction)getInfoPressed:(id)sender{
    
    [RKGetBluetoothInfoCommand sendCommand];

    if(robotstatus == 0){
        [messageView setString:[NSString stringWithFormat:@"%@ Connect to your Sphero.   (See help section for Bluetooth Pairing instructions.) \n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
    }
}

-(IBAction)executePressed:(id)sender {
    if( orbBasicProgram == nil ){
        [messageView setString:[NSString stringWithFormat:@"%@ You should save your program!\n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
        return;
    }
    if (robotstatus != 0) {
        [self clearDebug:nil];
        [self loadCurrentFileIntoOrbBasic];
        
        [orbBasicProgram execute];
    }
    else{
        [messageView setString:[NSString stringWithFormat:@"%@ Connect to your Sphero.   (See help section for Bluetooth Pairing instructions.) \n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
    }
}

-(IBAction)abortPressed:(id)sender {
    if (robotstatus != 0) {
        // Aborts the current program.
        [messageView setString:[NSString stringWithFormat:@"%@\n ------=  A B O R T  -  P R O G R A M  =------\n\n Resetting System Parameters:\n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
        
        [orbBasicProgram abort];
        
        // reset the state of the ball ie. Stabilization and BackLED and the UI.
        [self stabilizationOnPressed:nil];
        [stabilizationOnButton  setState:NSOnState];
        [stabilizationOffButton setState:NSOffState];

        [self backLEDOffPressed:nil];
        [backLEDOffButton setState:NSOnState];
        [backLEDOnButton  setState:NSOffState];
    }
}

-(IBAction)savePressed:(id)sender{
    [self openDialogToSave];
}

-(IBAction)sleepPressed:(id)sender{
    [RKGoToSleepCommand sendCommandWithDelay:0.25];
    
    [reconnectlabel setStringValue:@""];
    robotstatus = 0;

    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  sleep 0,0,0\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

// -------  Stabilization Buttons   ----------

-(IBAction)stabilizationOnPressed:(id)sender{
    [RKStabilizationCommand sendCommandWithState:RKStabilizationStateOn];

    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  ctrl = 1\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

-(IBAction)stabilizationOffPressed:(id)sender{
    [RKStabilizationCommand sendCommandWithState:RKStabilizationStateOff];
    
    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  ctrl = 0\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

// -------  BackLED Buttons   ----------

-(IBAction)backLEDOffPressed:(id)sender{
    [RKBackLEDOutputCommand sendCommandWithBrightness:0.0];

    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  backLED 0\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

-(IBAction)backLEDOnPressed:(id)sender{
    [RKBackLEDOutputCommand sendCommandWithBrightness:1.0];

    [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  backLED 255\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

// -------  Roll Buttons   ----------

-(IBAction)rollPressed:(id)sender{
    if ([stabilizationOffButton state] == NSOnState) {
        [messageView setString:[NSString stringWithFormat:@"%@ Stabilization is off! Turn it on to roll.\n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
        
    }
    else{
        [RKRollCommand sendCommandWithHeading:[directionSlider floatValue] velocity:[speedSlider floatValue]/255. stopped:false];
     
        [messageView setString:[NSString stringWithFormat:@"%@\nROLL:\n OrbBASIC command:  goroll %d, %d\n",[messageView string],[directionSlider intValue],[speedSlider intValue]]];
        [messageView setString:[NSString stringWithFormat:@"%@ OrbBASIC command:  delay %d\n",[messageView string],[delaySlider intValue]]];
        [messageView scrollToEndOfDocument:nil];
        
        [self performSelector:@selector(stopSphero) withObject:nil afterDelay:[delaySlider intValue]/1000.];
    }
}



#pragma mark Sliders

// -------  Adjust Heading Sliders   ----------
-(IBAction)adjustHeadingSliderMoved:(id)sender{
    int new_heading;
    
    if ([adjustHeadingSlider intValue] < 0) 
        new_heading = 360 + [adjustHeadingSlider intValue];
    
    else
        new_heading = 0 + [adjustHeadingSlider intValue];
    
    [RKRollCommand sendCommandWithHeading:new_heading velocity:0.0 stopped:false];
    [RKCalibrateCommand sendCommandWithDelay:1.0];
   
    [messageView setString:[NSString stringWithFormat:@"%@\nHEADING ADJUST:\n OrbBASIC command:  goroll %i,0\n OrbBASIC command:  delay 200\n OrbBASIC command:  heading 0\n\n",[messageView string],new_heading]];
    [messageView scrollToEndOfDocument:nil];
    
    if(robotstatus == 1)[[NSSound soundNamed:@"Pop"] play];
    
    // Reset the Slider to 0
    [adjustHeadingSlider setDoubleValue:0];
}


// -------  Roll Sliders   ----------
-(IBAction)speedSliderMoved:(id)sender{    
    [speedLabel setStringValue:[NSString stringWithFormat:@"%d",[speedSlider intValue]]];
}
-(IBAction)directionSliderMoved:(id)sender{
    [directionLabel setStringValue:[NSString stringWithFormat:@"%d˚",[directionSlider intValue]]];
}
-(IBAction)delaySliderMoved:(id)sender{
    [delayLabel setIntegerValue:[delaySlider intValue]];
}


// -------  RGB Sliders   ----------
-(IBAction)redSliderMoved:(id)sender{
    [redLabel setStringValue:[NSString stringWithFormat:@"%d",[redSlider intValue]]];
    [self updateColorWell];
}

-(IBAction)greenSliderMoved:(id)sender{
    [greenLabel setStringValue:[NSString stringWithFormat:@"%d",[greenSlider intValue]]];
    [self updateColorWell];
}

-(IBAction)blueSliderMoved:(id)sender{
    [blueLabel setStringValue:[NSString stringWithFormat:@"%d",[blueSlider intValue]]];
    [self updateColorWell];
}


#pragma mark Message Handlers


- (void)handleResponse:(RKDeviceResponse *)response
{
    if ([response isKindOfClass:[RKGetBluetoothInfoResponse class]]){
        if (response.code == RKResponseCodeOK ) {
            RKGetBluetoothInfoResponse* bluetoothResponse = (RKGetBluetoothInfoResponse*) response;
            
             NSMutableString *bluetoothAddress = [NSMutableString stringWithString:[bluetoothResponse address]];
            
            // Put the : in the address number for human readability
            for (int i = 1; i <= 5; i++) 
                [bluetoothAddress insertString:@":" atIndex:( (i*2) + i ) - 1];
            
            [reconnectlabel setStringValue:[bluetoothResponse name]];
            
            NSDictionary *attributes = [NSDictionary dictionaryWithObject:[NSColor colorWithCalibratedRed:[redSlider intValue]/255. green:[greenSlider intValue]/255. blue:[blueSlider intValue]/255. alpha:1.0f] forKey:NSForegroundColorAttributeName];
            NSAttributedString *attributedString = [[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"[ %@ ] ( %@ )\n",[bluetoothResponse name],bluetoothAddress] attributes:attributes] autorelease];
            
            [[messageView textStorage] appendAttributedString:attributedString];
            [messageView setFont:[NSFont fontWithName:@"Menlo" size:12]];
            [messageView scrollToEndOfDocument:nil];

            [RKRGBLEDOutputCommand sendCommandWithRed:[redSlider   intValue]/255. green:[greenSlider intValue]/255. blue:[blueSlider  intValue]/255.];
        }
    }
    
    if ([response isKindOfClass:[RKGetPowerStateResponse class]]){
        if (response.code == RKResponseCodeOK ) {
            RKGetPowerStateResponse* newPowerState = (RKGetPowerStateResponse*) response;
            
            NSDictionary *attributes = [NSDictionary dictionaryWithObject:[NSColor colorWithCalibratedRed:(1.0 - (([newPowerState batteryVoltage] - 7.29))) green:([newPowerState batteryVoltage] - 7.29) blue:0.0 alpha:1.0f] forKey:NSForegroundColorAttributeName];
            NSAttributedString *attributedString = [[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\t\t\t\t\t%0.2f volts  ~%d%%\n\n",[newPowerState batteryVoltage],(int)((([newPowerState batteryVoltage] - 7.0) / 1.29) * 100)] attributes:attributes] autorelease];
            
            [[messageView textStorage] appendAttributedString:attributedString];
            [messageView setFont:[NSFont fontWithName:@"Menlo" size:12]];
            [messageView scrollToEndOfDocument:nil];
        }
    }
    
    // Check Responses
    if ([response isKindOfClass:[RKOrbBasicAppendFragmentResponse class]]) {
        if (response.code == RKResponseCodeErrorParameter) {
            NSDictionary *attributes = [NSDictionary dictionaryWithObject:[NSColor colorWithCalibratedRed:1.0 green:0.0 blue:0.0 alpha:1.0f] forKey:NSForegroundColorAttributeName];
            
            NSAttributedString *attributedString = [[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@" Fail!:  Syntax Error\n"] attributes:attributes] autorelease];
            
            [[messageView textStorage] appendAttributedString:attributedString];
            [messageView setFont:[NSFont fontWithName:@"Menlo" size:12]];
        } else if (response.code == RKResponseCodeErrorExecute) 
            [messageView setString:[NSString stringWithFormat:@"%@ Fail!:  Memory Full!\n",[messageView string]]];
        else if( response.code == RKResponseCodeOK ) 
            [messageView setString:[NSString stringWithFormat:@"%@ Loading and Executing Program. ->  \n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
    }
    else if ([response isKindOfClass:[RKOrbBasicEraseStorageResponse class]]) {
        if( response.code == RKResponseCodeOK )
            [messageView setString:[NSString stringWithFormat:@"%@ Erasing Program Running on Sphero.   Success.\n",[messageView string]]];
        else
            [messageView setString:[NSString stringWithFormat:@"%@ Fail!: Error Erasing Program Running on Sphero.\n",[messageView string]]];
        [messageView scrollToEndOfDocument:nil];
    }
}

- (void)handleAsyncData:(RKDeviceAsyncData *)data
{
    if ([data isKindOfClass:[RKOrbBasicPrintMessageData class]]) {
        RKOrbBasicPrintMessageData *printMessage = (RKOrbBasicPrintMessageData *)data;
        [messageView setString:[NSString stringWithFormat:@"%@\nSPHERO->  %@",[messageView string],printMessage.message]];
    } else if ([data isKindOfClass:[RKOrbBasicErrorASCIIData class]]) {        
        RKOrbBasicErrorASCIIData *errorMessage = (RKOrbBasicErrorASCIIData *)data;
        [messageView setString:[NSString stringWithFormat:@"%@\nSPHERO ?->  %@\n",[messageView string],errorMessage.error]];
    } else if ([data isKindOfClass:[RKOrbBasicErrorBinaryData class]]) 
        [messageView setString:[NSString stringWithFormat:@"%@ Fail!: OrbBasic Binaray Error.  END.\n",[messageView string]]];
    [messageView scrollToEndOfDocument:nil];
}

#pragma Help Examples

-(IBAction)showExampleWithText:(NSString *)text{
    
    if (orbBasicProgram != nil) {
        // Save the current  into the filename.
        [[[orbbasic string] dataUsingEncoding:NSUTF8StringEncoding] writeToFile:[[filePathlabel stringValue] stringByAppendingPathComponent:[filenamelabel stringValue]] atomically:YES];
        orbBasicProgram = nil;
        [filenamelabel setStringValue:@""];
        [orbbasic setString:@""];
    }
    
    NSString *documentData = [NSString stringWithFormat:@""];
    NSString *replacementText = [NSString stringWithFormat:@""];
    
    [messageView setString:[NSString stringWithFormat:@"-> Loaded Example for: %@\n\n", text]];
    [messageView scrollToEndOfDocument:nil];
    
    // -------------  VARIABLES -------------------
    if ([text isEqualToString: @"ctrl"]){
        replacementText = @"10 ctrl = 1\n20 print \"Control system is \",ctrl,;\n30 if ctrl = 1 then ?\"(on)\" else ?\"(off)\"";
        documentData = [NSString stringWithFormat:@"When read, this variable returns 0 if the control system if off and 1 if it is on. When assigned, 0 turns off the control system and any other value turns it on.\n\n"];
    }
    else if ([text isEqualToString:@"speed"]){
        replacementText = @"10 ctrl = 1\n20 LEDC 8\n30 X = speed\n40 backLED X\n50 goto 30";
        documentData = [NSString stringWithFormat:@"Returns the approximate speed of the robot in real time. This value is the filtered average speed of both motors so there is some delay between what you see and what this returns. It is read-only and ranges from 0..255.\n\n"];
    }
    else if ([text isEqualToString: @"TimerA, TimerB, TimerC"]){
        replacementText = @"10 timerB = 32000\n20 if speed > 0 then goto 20\n30 A = (32000-timerB)/1000\n40 if A = 0 then print \"Never stopped!\" else print A \" seconds to stop\"";
        documentData = [NSString stringWithFormat:@"These are built-in timers that count down to zero by themselves, once per millisecond. They are both readable and writeable. The program will tell you if you stopped within 32 seconds of running the program. \n\n"];
    }
    else if ([text isEqualToString: @"spdval, hdgval"]){
        replacementText = @"10 if cmdval > 0 then ?\"New heading, speed:\" hdgval, spdval \n20 delay 100      \n30 goto 10";
        documentData = [NSString stringWithFormat:@"Returns the last commanded speed portion of a roll command sent from the smartphone client, ranging from 0 (stop) to 255 (full speed). \n\nReturns the last command heading portion of a roll command sent from the smartphone client ranging from 0 to 359 degrees.\n\n"];
    }
    else if ([text isEqualToString: @"isconn"]){
        replacementText = @"10 if isconn > 0 then LEDC 1 else LEDC 2\n20 delay 100\n30 goto 10";
        documentData = [NSString stringWithFormat:@"This variable provides real time access to the current state of a connected smartphone over Bluetooth. \n\n"];
    }
    else if ([text isEqualToString: @"dshake"]){
        replacementText = @"10 if dshake > 0 then LEDC 1 else LEDC 2\n20 delay 100\n30 goto 10";
        documentData = [NSString stringWithFormat:@"Returns a value of 1 when double-shake event has been detected. Reading this system variable automatically sets it to zero. \n\n"];
    }
    else if ([text isEqualToString: @"accelone"]){
        replacementText = @"10 ctrl = 0\n20 if accelone < 200 then ? \"Free fall! (\" accelone \")\"\n30 if accelone > 900 then ?\"Still\"\n40 delay 200\n50 goto 20";
        documentData = [NSString stringWithFormat:@"Returns the effective acceleration vector that Sphero is experiencing. The IMU computes this as the vector sum of the X, Y and Z axis components so it is expressed here multiplied by 1,000. A perfectly calibrated Sphero will return 1,000 when still, 0 when in free fall and up to 8,000 when being shaken. \n\n Run this, and TOSS YOUR SPHERO! \n\n"];
    }
    else if ([text isEqualToString: @"xpos, ypos"]){
        replacementText = @"10 locate 0,0\n20 delay 100\n30 R = sqrt (xpos*xpos + ypos*ypos)\n40 if R < 20 then LEDC 1 else LEDC 2\n50 goto 20";
        documentData = [NSString stringWithFormat:@"Returns the current x and y position of Sphero as determined by the internal locator. The approximate scale is in centimeters. Use the function locate to assign these values. \n\nRun this program in the background while using the Drive App to move Sphero in and out of the 40cm circle. \n\n"];
    }
    else if ([text isEqualToString: @"roll, pitch, yaw"]){
        replacementText = @"10 ctrl = 0\n20 print pitch, roll, yaw\n30 delay 100\n40 goto 20";
        documentData = [NSString stringWithFormat:@"Here is a good program to display the relationship of pitch, roll and yaw \n\n"];
    }
    else if ([text isEqualToString: @"accelX, accelY, accelZ"]){
        replacementText = @"10 ctrl = 0\n20 print accelX, accelY, accelZ\n30 delay 100\n40 goto 20";
        documentData = [NSString stringWithFormat:@"Returns the current filtered reading from the accelerometer. The range is -32768 to +32767 for full- scale, which defaults to ±8Gs. The convention is that X is the pitch axis, Y is roll and Z is yaw. \n\n"];
    }
    else if ([text isEqualToString: @"gyroX, gyroY, gyroZ"]){
        replacementText = @"10 ctrl = 0\n20 print gyroX, gyroY, gyroZ\n30 delay 100\n40 goto 20";
        documentData = [NSString stringWithFormat:@"Returns the current filtered reading from the rate gyro in units of 0.1 degrees/second. The range is - 20000 to +20000 for full-scale. The convention is that X is the pitch axis, Y is roll and Z is yaw. \n\n"];
    }
    
    // -------------  HELLO SPHERO -------------------
    else if ([text isEqualToString: @"hello sphero"]){
        replacementText = @"10 print \"Hello Sphero\"\n20 RGB 0, 0, 255\n30 delay 1500\n40 end";
        documentData = [NSString stringWithFormat:@"Print \"Hello Sphero\" in the debug window and turn Sphero Blue for 1.5 Seconds. \n\n"];
    }
    
    // -------------  FUNCTIONS -------------------
    else if ([text isEqualToString: @"sqrt x"]){
        replacementText = @"10 for X = 10 to 100 step 10\n20 Q = sqrt X*10000\n30 print \"sqrt(\" X \") = \" Q/100 \".\" Q % 100\n40 next X";
        documentData = [NSString stringWithFormat:@"Since orbBasic only supports integer math you need to understand the following identity to really use this for smaller numbers:\n\n√(𝑎 ∙ 𝑏)   = √𝑎   ∙  √𝑏  \nConsider the following example where you want to take the square root of 42:\n\nLet a = 42 and b = 10000, so sqrt 420000 yields 648. Since √100000 equals 100, this answer is 100 times too large. Which lines up with the real answer of √42 which is about 6.48 \n\n"];
    }
    else if ([text isEqualToString: @"rnd x"]){
        replacementText = @"10 for X = 1 to 50\n20 RGB rnd 255,rnd 255,rnd 255\n30 delay rnd 250\n40 next X";
        documentData = [NSString stringWithFormat:@"This returns a pseudo random number between 1 and the value provided, which can be either a literal number or an expression. Use the random statement to reseed the number generator. \n\n"];
    }
    else if ([text isEqualToString: @"abs x"]){
        replacementText = @"10 A = -20\n20 print A\n30 A = abs A\n40 print A\n50 end";
        documentData = [NSString stringWithFormat:@"This returns the absolute value of an expression, which is always positive. \n\n"];
    }
    
    // -------------  COMMANDS -------------------
    else if ([text isEqualToString: @"print"]){
        replacementText = @"10 for X = 65 to 67\n20 print X \" as a number, \" $X \" as a char \"\n30 next X";
        documentData = [NSString stringWithFormat:@"The final length of each print statement is limited to 32 bytes. ( If you exceed this, a warning message is printed instead.) The print statement is a sequence of string literals and expressions. String literals get copied exactly as written while expressions are evaluated and their numeric value is expressed.\n\nif the $ symbol precedes an expression, it is evaluated but instead of the number itself a single character, the ASCII value of the expression, is emitted instead. Commas between print elements insert a space in the output. If the final character of the print string is terminated with a semicolon then no linefeed is appended to the message sent back to the client.\n\nThe output of print is enqueued for transmit so it is possible to fill up that queue faster than it can be emptied over the Bluetooth link. If this happens, data is not lost but program execution is paused until the print statement can complete normally.\n\n The single ? character is shorthand for print and can help save program space. \n\n"];
    }
    else if ([text isEqualToString: @"delay x"]){
        replacementText = @"10 for X = 1 to 74\n20 RGB X,X,X\n30 delay X*4\n40 RGB 0,0,0\n50 delay X{1\n60 next X";
        documentData = [NSString stringWithFormat:@"This is a built-in delay statement that doesn't use the three timers. It takes one parameter, an expression resolving to the number of milliseconds in the delay. \n\n"];
    }
    else if ([text isEqualToString: @"RGB r,g,b"]){
        replacementText = @"10 for A = 0 to 255\n20 RGB A, 255-A, A\n30 delay 100\n40 next A";
        documentData = [NSString stringWithFormat:@"This commands the RGB LED with the three provided parameters, each of which is evaluated as an expression. Values range from 0,0,0 (off) to 255,255,255 (white). Note that this statement is in all-caps since its still an abbreviation. The RGB LED is updated every 1ms so changing its value faster than this with orbBasic is ineffective. \n\n"];
    }
    else if ([text isEqualToString: @"LEDC x"]){
        replacementText = @"10 for X = 1 to 8\n20 LEDC X\n30 delay 100\n40 next X";
        documentData = [NSString stringWithFormat:@"This is like the RGB command but sets the LED to one of eight predefined colors, without requiring you to know the three component values. \n\n 0 <off> | 1 red | 2 green | 3 blue | 4 orange | 5 purple | 6 white | 7 yellow | 8 <off>\n\n"];
    }
    else if ([text isEqualToString: @"backLED x"]){
        replacementText = @"10 X = 1\n20 backLED X\n30 delay 50\n40 X = X { 1\n50 if X < 256 then goto 20\n60 goto 10";
        documentData = [NSString stringWithFormat:@"This commands the aiming LED with an intensity from 0..255 \n\n"];
    }
    else if ([text isEqualToString: @"goroll h,s,g"]){
        replacementText = @"10 goroll 0,128,2\n20 delay 500\n30 goroll 0,0,0\n40 delay 255\n50 goroll 90,128,2\n60 delay 500\n70 goroll 0,0,0\n80 RGB 255,0,255\n90 delay 255";
        documentData = [NSString stringWithFormat:@"This executes a roll command to the control system with the provided values of heading and speed. Heading is over the usual range of 0..359 and speed over 0..255. The last parameter, a flag for \"go\" tells the control system whether to force fast rotate to on (g=2), attempt to drive normally (g = 1) or perform a controlled stop (g=0). \n\n"];
    }
    else if ([text isEqualToString: @"heading h"]){
        replacementText = @"10 LEDC 4\n15 A = 0\n20 goroll 0,128,2\n30 delay 1000\n40 goroll 0,0,0\n50 A = A + 90\n60 heading A\n70 if A = 270 then goto 90\n80 goto 20\n90 end";
        documentData = [NSString stringWithFormat:@"This assigns the current yaw heading to the provided value, in the range of 0..359. \n\n"];
    }
    else if ([text isEqualToString: @"raw Lm, Ls, Rm, Rs"]){
        replacementText = @"10 LEDC 3\n20 raw 1,255,2,255\n30 delay 2000\n40 raw 0,0,0,0\n50 delay 500\n60 ctrl = 1\n70 end";
        documentData = [NSString stringWithFormat:@"raw LeftMode,LeftSpeed,RightMode,RightSpeed\n\nThis allows you to take over one or both of the motor output values, instead of having the stabilization system control them. Each motor (left and right) requires a mode (see below) and a power value from 0- 255. This command will disable stabilization if both modes aren't \"ignore\" so you'll need to re-enable it once you're done.\n\n MODES: \n | 0 Off  | 1 Forward  | 2 Reverse  | 3 Brake  | 4 Ignore |\n\n"];
    }
    else if ([text isEqualToString: @"data, read"]){
        replacementText = @"10 data 10,20,30\n20 read A,B\n30 rstr\n40 read C\n50 print A,B,C\n60 data 100\n70 read A\n80 print A,B,C";
        documentData = [NSString stringWithFormat:@"data d1{d2,d3...}\nDefines a set of constant data values that can be sequentially read via the read statement. You can specify more than the maximum number of values (currently set to 25) and they will be ignored.  \n\n read X{Y,...}\nReads the next value(s) from the current data set into the specified variable(s). If you attempt to read past the end, you will get an error.\n\n"];
    }
    else if ([text isEqualToString: @"locate x,y"]){
        replacementText = @"10 locate 0,0\n20 goroll 0,100\n30 delay 3000\n40 goroll 0,0\n50 delay 1000\n60 print xpos,ypos";
        documentData = [NSString stringWithFormat:@"This assigns the XY position of the internal locator to the provided values. \n\n"];
    }
    NSDictionary *attributes = [NSDictionary dictionaryWithObject:[NSColor colorWithCalibratedRed:0.7 green:0.4 blue:9.0 alpha:1.0f] forKey:NSForegroundColorAttributeName];
    NSAttributedString *attributedString = [[[NSAttributedString alloc] initWithString:documentData attributes:attributes] autorelease];
    [[messageView textStorage] appendAttributedString:attributedString];
    [messageView setFont:[NSFont fontWithName:@"Menlo" size:12]];
    [messageView scrollToEndOfDocument:nil];
    
    [orbbasic setString: replacementText];
}

@end
